! #############################################################################
!
! HW3Dv12.f95
! - Compiled as project in Plato 3 along with dnsqe.f and dnsq.f
! 
! 
! 
! #############################################################################

MODULE variables

IMPLICIT NONE

REAL, ALLOCATABLE, DIMENSION(:)     	:: pitot_E         			! ALLOCATED in read_calibration_data, DEALLOCATED in
REAL, ALLOCATABLE, DIMENSION(:)     	:: wire_E1					! read_effective_angle_calibration_data and
REAL, ALLOCATABLE, DIMENSION(:)     	:: wire_E2					! read_velocity_calibration_data depending on the calibration 
REAL, ALLOCATABLE, DIMENSION(:)     	:: wire_E3					! procedure

INTEGER                                 :: no_vel_cal_points    	! Number of datapoints in velocity calibration
INTEGER                                 :: no_angle_cal_points  = 9	! Number of points in effective angle calibration
INTEGER    								:: total_no_of_vel_points   ! If a second zero measurement is taken

REAL, DIMENSION(3)                      :: e_0						! Zero velocity voltage for the wires

REAL, ALLOCATABLE, DIMENSION(:,:)     	:: e_vel_cal				! Built in read_velocity_calibration_data
REAL, ALLOCATABLE, DIMENSION(:,:,:)   	:: e_ea_cal					! Effective angle voltage data
REAL, ALLOCATABLE, DIMENSION(:) 		:: vel_cal_pitot_E			! Voltages for transducer, velocity calibration
REAL, ALLOCATABLE, DIMENSION(:,:) 		:: ea_cal_pitot_E			! Voltages for transducer, effective angle calibration


REAL                                    :: pitot_min_E				! Stores transducer zero voltage
INTEGER									:: order_of_polynomial		! Order of polynomial fit

! Temperatures
REAL, ALLOCATABLE, DIMENSION(:) 		:: vel_cal_T_array			! Temperature data for velocity calibration
REAL, ALLOCATABLE, DIMENSION(:,:) 		:: ea_cal_T_array			! Temperature data for angle calibration
REAL, ALLOCATABLE, DIMENSION(:)     	:: traverse_T_array   		! Temperature data for traverse
REAL                                    :: T_0						! Reference temperature
REAL				                    :: T_w						! Wire temperature

! Temperature corrected arrays
REAL, ALLOCATABLE, DIMENSION(:,:) 		:: corr_e_vel_cal	! Temperature corrected voltage data for velocity calibration 
REAL, ALLOCATABLE, DIMENSION(:,:,:) 	:: corr_e_ea_cal	! Temperature corrected voltage data for angle calibration

! Transducer and anemometer constants
REAL 									:: K_pitot_1  		! Pitot calibration factor, factor
REAL 									:: K_pitot_2  		! Pitot calibration factor, zero offset
INTEGER 								:: pitot_gain    	! Amplifier gain
REAL									:: K1,K2,K3  		! Anemometer amplification 
REAL, DIMENSION(3)                      :: eofz 


REAL, ALLOCATABLE, DIMENSION(:) 		:: pitot_U
REAL, ALLOCATABLE, DIMENSION(:,:) 		:: pitot_U_ea
REAL, ALLOCATABLE, DIMENSION(:) 		:: vel_pitot_U
REAL, DIMENSION(3,5)     				:: coeff_mat
REAL, ALLOCATABLE, DIMENSION(:,:,:) 	:: U_hw_angle_mat
REAL, DIMENSION(3)                      :: eff_angle
REAL, DIMENSION(3) 						:: U_hw

! Other constants
REAL                                    :: pi = 3.141592654
REAL 									:: rho = 1.2  		! Air density
REAl, DIMENSION(9)    					:: calibration_angles = (/-20.,-15.,-10.,-5.,0.,5.,10.,15.,20./)
REAL									:: phi_w1 = 90.

! Filenames:
CHARACTER(len=70)      					:: angle_calibration_folder
CHARACTER(len=70)       				:: timeseries_folder
CHARACTER(len=70)      					:: filename_velocity_calibration 
CHARACTER(len=70)       				:: filename_angle_calibration_base 
CHARACTER(len=70)       				:: filename_velocity_calibration_temperatures
CHARACTER(len=70)       				:: filename_angle_calibration_temperatures
CHARACTER(len=70)       				:: filename_settings
CHARACTER(len=70)       				:: filename_velocity_calibration_log
CHARACTER(len=70)       				:: filename_eff_angle_calibration_log
CHARACTER(len=70)       				:: filename_log 
CHARACTER(len=70)       				:: filename_statistics
CHARACTER(len=70)       				:: filename_polyfit
CHARACTER(len=70)       				:: filename_meanprofile
CHARACTER(len=70)       				:: filename_meanprofile_statistics
CHARACTER(len=70)       				:: filename_conditional_average
CHARACTER(len=70)       				:: filename_stored_velocity_calibration 
CHARACTER(len=70)       				:: filename_stored_angles
CHARACTER(len=70)       				:: filename_timeseries
CHARACTER(len=70)       				:: filename_timeseries_temperature
CHARACTER(len=70)       				:: filename_U_vector_base


! Filedata
INTEGER									:: vel_no_headers 
INTEGER									:: angle_no_headers
INTEGER									:: vel_start_line 
INTEGER									:: vel_end_line 
INTEGER									:: vel_number_of_lines
INTEGER									:: zero_line

! Data on timeseries
INTEGER                                 :: timeseries_no_points
INTEGER                                 :: timeseries_sampling_rate
INTEGER                                 :: timeseries_sampling_time
INTEGER                                 :: timeseries_line_start
     		
INTEGER 								:: no_invalid_points

END MODULE variables


! #############################################################################

MODULE routines2

IMPLICIT NONE

CONTAINS

! #############################################################################

SUBROUTINE FCN(N,U,FVEC,IFLAG)

USE variables

! Calculate the functions at X and return this vector in FVEC. Must be external.

! N - Number of unknowns
! U - Value of U to check for
! FVEC - Result
! IFLAG - Not used, see DNSQE

INTEGER                         :: N,IFLAG,i
DOUBLE PRECISION                :: U(N),FVEC(N)
DOUBLE PRECISION, DIMENSION(3)  :: phi 
DOUBLE PRECISION, DIMENSION(3)  :: alpha_e 

alpha_e = eff_angle

phi = (/phi_w1,phi_w1 + 239.5,phi_w1 + 125./)*(3.14159/180.) ! No3

do i = 1,N

    FVEC(i) = ((U(1)**2) + (U(2)*COS(phi(i)) + U(3)*SIN(phi(i)))**2)*(COS(alpha_e(i) + &
                     ATAN( ( U(2)*COS(phi(i)) + U(3)*SIN(phi(i)) )/U(1) )))**2 + &
                     (U(2)*SIN(phi(i)) - U(3)*COS(phi(i)))**2 - &
                     (U_hw(i)*COS(alpha_e(i)))**2
              
end do
    
END SUBROUTINE FCN

! #############################################################################


END MODULE routines2

! #############################################################################

SUBROUTINE settings(filename)

! Reads in the current settings

USE variables

CHARACTER(len=70)   	:: filename
CHARACTER(len=70)   	:: dummy

OPEN (UNIT=4, FILE=filename, STATUS='old', ACTION='READ' &
  	 	&, ACCESS='sequential')

 READ(4,*) dummy  ! Pressure transducer amplification
 READ(4,*) pitot_gain
 READ(4,*) dummy  ! Pressure transducer calibration constant
 READ(4,*) K_pitot_1
 READ(4,*) K_pitot_2
 READ(4,*) dummy  ! Anemometer amplification     
 READ(4,*) K1
 READ(4,*) K2
 READ(4,*) K3
 READ(4,*) dummy  ! Wire operating temperature    
 READ(4,*) T_w
 READ(4,*) dummy  ! Hotwire eofz   
 READ(4,*) eofz(1)
 READ(4,*) eofz(2)
 READ(4,*) eofz(3)
 READ(4,*) dummy  ! Velocitycalibration(#headerlines, number of lines,start,end)
 READ(4,*) vel_no_headers
 READ(4,*) vel_number_of_lines
 READ(4,*) vel_start_line 
 READ(4,*) vel_end_line 
 READ(4,*) zero_line 
 READ(4,*) dummy  ! Angle calibration(#headerlines, expects 9 lines)
 READ(4,*) angle_no_headers
	
CLOSE(4)

END SUBROUTINE

! #############################################################################

MODULE routines

IMPLICIT NONE

CONTAINS

! #############################################################################

SUBROUTINE filedata()

! Reads in current filedata

USE variables

CHARACTER(len=70)      					:: dummy

OPEN (UNIT=4, FILE='indata/filedata.txt', STATUS='unknown', ACTION='READ' &
     &, ACCESS='sequential')


READ(4,*) dummy  ! Angle calibration folder
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') angle_calibration_folder
READ(4,*) dummy  
READ(4,*) dummy  ! Timeseries folder
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') timeseries_folder
READ(4,*) dummy  
READ(4,*) dummy  ! Settings filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_settings
READ(4,*) dummy  
READ(4,*) dummy  ! Velocity calibration filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_velocity_calibration
!write(*,*) filename_velocity_calibration
READ(4,*) dummy  
READ(4,*) dummy  ! Effective angle calibration filename base    
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_angle_calibration_base
READ(4,*) dummy 
READ(4,*) dummy  ! Velocity calibration temperatures filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_velocity_calibration_temperatures
READ(4,*) dummy
READ(4,*) dummy  ! Effective angle calibration temperatures filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_angle_calibration_temperatures
READ(4,*) dummy
READ(4,*) dummy  ! Velocity calibration log filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_velocity_calibration_log
READ(4,*) dummy
READ(4,*) dummy  ! Angle calibration log filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_eff_angle_calibration_log
READ(4,*) dummy
READ(4,*) dummy  ! Log filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_log
READ(4,*) dummy
READ(4,*) dummy  ! Stored angle calibration filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_stored_angles
READ(4,*) dummy
READ(4,*) dummy  ! Stored velocity calibration filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_stored_velocity_calibration
READ(4,*) dummy
READ(4,*) dummy  ! Timeseries filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_timeseries
READ(4,*) dummy
READ(4,*) dummy  ! Timeseries temperature filename
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_timeseries_temperature
READ(4,*) dummy
READ(4,*) dummy  ! Timeseries data
READ(4,*) dummy  ! -------------------------------------------------
READ(4,*) timeseries_sampling_rate
READ(4,*) timeseries_sampling_time
READ(4,*) timeseries_no_points
READ(4,*) timeseries_line_start
READ(4,*) dummy
READ(4,*) dummy  ! U_vector filename base
READ(4,*) dummy  ! -------------------------------------------------
READ(4,'(A70)') filename_U_vector_base
READ(4,*) dummy
READ(4,*) dummy  ! Timeseries statistics
READ(4,*) dummy  ! ------------------------------------------------------------
READ(4,'(A70)') filename_statistics
READ(4,*) dummy
READ(4,*) dummy  ! Polyfit plotdata
READ(4,*) dummy  ! ------------------------------------------------------------
READ(4,'(A70)') filename_polyfit
READ(4,*) dummy
READ(4,*) dummy  ! Meanprofile filename
READ(4,*) dummy  ! ------------------------------------------------------------
READ(4,'(A70)') filename_meanprofile
READ(4,*) dummy
READ(4,*) dummy  ! Meanprofile statisitics filename
READ(4,*) dummy  ! ------------------------------------------------------------
READ(4,'(A70)') filename_meanprofile_statistics
READ(4,*) dummy
READ(4,*) dummy  ! Conditional average filename
READ(4,*) dummy  ! ------------------------------------------------------------
READ(4,'(A70)') filename_conditional_average


CLOSE(4)

END SUBROUTINE filedata
 
! #############################################################################

!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$   Velocity calibration  !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
 
! #############################################################################

SUBROUTINE read_velocity_calibration_data(filename)

! Reads velocity calibration data.
! Input: filename
! Allocates: e_vel_cal(3,no_vel_cal_points) -> anemometer output
!            vel_cal_pitot_E(no_vel_cal_points) 
! Defines global variables: e_0(3) -> anemometer output at zero velocity
!         					pitot_min_E -> amplified output from pressure transducer at zero velocity
!          					no_vel_cal_points

USE variables

CHARACTER(len=70)   	:: filename
INTEGER                 :: number_of
INTEGER    				:: i,k

!write(*,*) filename

CALL read_calibration_data(filename,number_of,vel_no_headers)  ! Reads calibration data

no_vel_cal_points = vel_end_line - vel_start_line + 1   ! Removes zero point
total_no_of_vel_points = number_of   ! Used to check drift between first and last zero measurement

ALLOCATE(e_vel_cal(3,no_vel_cal_points));
ALLOCATE(vel_cal_pitot_E(no_vel_cal_points));

WRITE(*,*) 'Zero velocity voltage, E, wire 1-3'
WRITE(*,*) wire_E1(zero_line)
WRITE(*,*) wire_E2(zero_line)
WRITE(*,*) wire_E3(zero_line)
WRITE(*,*)

WRITE(*,*) 'Offset voltage, eofz, wire 1-3'
WRITE(*,*) eofz(1)
WRITE(*,*) eofz(2)
WRITE(*,*) eofz(3)
WRITE(*,*)

e_0(1) = (wire_E1(zero_line)/K1) - eofz(1)
e_0(2) = (wire_E2(zero_line)/K2) - eofz(2)
e_0(3) = (wire_E3(zero_line)/K3) - eofz(3)

WRITE(*,*) 'Zero voltage, e_0, wire 1-3'
WRITE(*,*) e_0(1)
WRITE(*,*) e_0(2)
WRITE(*,*) e_0(3)
WRITE(*,*)

WRITE(*,*) 'Calibration data, E, wire 1-3'

k = 0

do i = vel_start_line,vel_end_line

  	WRITE(*,*) wire_E1(i),wire_E2(i),wire_E3(i)

	k = k + 1
  
	e_vel_cal(1,k) = (wire_E1(i)/K1) - eofz(1)
	e_vel_cal(2,k) = (wire_E2(i)/K2) - eofz(2)
	e_vel_cal(3,k) = (wire_E3(i)/K3) - eofz(3)
    
end do

WRITE(*,*)

vel_cal_pitot_E = pitot_E(vel_start_line:vel_end_line)

pitot_min_E = pitot_E(zero_line)

WRITE(*,*) 'Zero velocity pitot voltage'
WRITE(*,*) pitot_min_E
WRITE(*,*)

DEALLOCATE(wire_E1,wire_E2,wire_E3, pitot_E)

END SUBROUTINE


! #############################################################################

SUBROUTINE read_calibration_data(filename, number_of,no_headers) 

! General dataimport subroutine, used by both read_velocity_calibration_data and 
! read_effective_angle_calibration_data.
! Input: filename
! Output: number_of
! Allocates: pitot_E,wire_E1,wire_E2,wire_E

USE variables

INTEGER												:: no_headers
INTEGER      			 							:: status
INTEGER                                				:: i
INTEGER                           				    :: number_of
CHARACTER(len=70),INTENT(IN) 						:: filename
CHARACTER(len=50) 				 					:: test
INTEGER 											:: readstatus

DEALLOCATE(pitot_E,wire_E1,wire_E2,wire_E3)

! Open input file with logged ramp signal and hw signals
OPEN (UNIT=100, FILE=filename, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

! Read headers

do i = 1,no_headers-1
	READ(100,*,IOSTAT = readstatus) test
end do

!write(*,*) 'here'

1001 FORMAT (4F8.3)

number_of = 0

! Find number of datapoints in file
   
do 

	READ(100,1001,IOSTAT = readstatus) test

    if (readstatus==-1) exit   ! Checks for end of file
    
    number_of = number_of + 1

end do

100 FORMAT(/,4X, 'Number of data points in calibration file is ', I7)

WRITE(*,100) number_of

ALLOCATE(pitot_E(number_of),wire_E1(number_of),wire_E2(number_of),wire_E3(number_of))

REWIND(UNIT=100)

do i = 1,no_headers-1
	READ(100,*,IOSTAT = readstatus) test
end do

do i = 1,number_of
      
    READ(100,*,IOSTAT = readstatus) wire_E1(i), wire_E2(i), wire_E3(i), pitot_E(i)

    WRITE(*,1001) wire_E1(i), wire_E2(i), wire_E3(i), pitot_E(i)
    	       
	if (readstatus==-1) exit   ! Checks for end of file
    
end do

CLOSE(100)

END Subroutine

! #############################################################################

SUBROUTINE vel_cal_temperatures(filename)

! Subroutine used to store temperatures for correction.

! Allocates: vel_cal_T_array(no_vel_cal_points)
! Defines: T_0 -> reference temperature

USE variables

CHARACTER(len=70)					:: filename
INTEGER								:: status,i,k
REAL, ALLOCATABLE, DIMENSION(:)     :: dummyarray

k = 0

ALLOCATE(dummyarray(total_no_of_vel_points))
ALLOCATE(vel_cal_T_array(no_vel_cal_points))


!write(*,*) filename

OPEN (UNIT=100, FILE=filename, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

do i = 1,total_no_of_vel_points

	READ(100,*) dummyarray(i)

end do 

WRITE(*,*) 'Temperature for calibration points'

do i = vel_start_line, vel_end_line

  	WRITE(*,*) dummyarray(i)

	k = k + 1

    vel_cal_T_array(k) = dummyarray(i)
	!write(*,*) vel_cal_T_array(k)
    

end do

WRITE(*,*)
  
CLOSE(100)

T_0 = dummyarray(zero_line)

WRITE(*,*) 'Reference temperature, T_0'
WRITE(*,*) T_0
WRITE(*,*)

END SUBROUTINE 

! #############################################################################

SUBROUTINE temperature_corrections(choice)

! Corrects data for temperature change
! Input: choice -> specifies if it is the velocity or angle data which should be corrected

USE variables

INTEGER    :: choice                 ! 1 = velocity calibration, 2 = angle calibration
INTEGER    :: i,j,l								  
REAL	   :: tfak


WRITE(*,*) 'Reference temperature, T_0'
WRITE(*,*) T_0
WRITE(*,*)


if(choice == 1) then  ! Uses temperature data for velocity calibration 

 	ALLOCATE(corr_e_vel_cal(3,no_vel_cal_points))

 	do i = 1,3
 
     	do j = 1,no_vel_cal_points

        	tfak = ((T_w - T_0)/(T_w - vel_cal_T_array(j)))**0.5

	        corr_e_vel_cal(i,j) = e_vel_cal(i,j)*tfak
            
 
     	end do
 
 	end do

elseif(choice == 2) then   ! Uses temperature data for angle calibration 

 	ALLOCATE(corr_e_ea_cal(3,no_angle_cal_points,3))

	rel_to: do l = 1,3

    	wire: do i = 1,3

    		angle: do j = 1,no_angle_cal_points

				tfak = sqrt((T_w - T_0)/(T_w - ea_cal_T_array(l,j)))
    
         		corr_e_ea_cal(i,j,l) = e_ea_cal(i,j,l)*tfak
             
     		end do
             
 		end do

    end do


end if
    
END SUBROUTINE

! #############################################################################

SUBROUTINE pitot_E2U(E_values,no_points)

! Converts the amplified transducer voltage to velocity
! Input: E_values, no_points
! Allocates: pitot_U

USE variables

INTEGER        				 :: i
INTEGER         			 :: no_points
REAL, DIMENSION(no_points)   :: E_values

ALLOCATE(pitot_U(no_points))

WRITE(*,*) 'Zero velocity transducer voltage'
WRITE(*,*) pitot_min_E
WRITE(*,*)
WRITE(*,*) 'Transducer voltage'

do i = 1,no_points

  	WRITE(*,*) E_values(i)
    
	pitot_U(i) = (((((E_values(i) - pitot_min_E)/pitot_gain)*K_pitot_1 + K_pitot_2 )*2)/rho)**0.5

end do

WRITE(*,*)

END SUBROUTINE

! #############################################################################

SUBROUTINE find_polynomials()

! Writes input data to INN.DAT and calls kurve-mac2.exe which reads the datafile, curvefits
! the data and writes the polynomial coefficients to KURVE_OUT.DAT. The coefficients are then 
! read from KURVE_OUT.DAT by the subroutine.
! Defines: coeff_mat

USE variables

REAL, DIMENSION(5)      :: initial_value = (/0.2,0.2,0.2,0.2,0.2/)
INTEGER                 :: status, readstatus,i,j

1000 FORMAT (I1)

1001 FORMAT (F4.2,',',F4.2,',',F4.2,',',F4.2,',',F4.2)

1002 FORMAT (F8.3, F8.3)

outer: do i = 1,3
	
	WRITE(*,400) i
	400 FORMAT( /,4X, 'Calibrating for wire ', I1 )

    WRITE(*,401) 
    401 FORMAT( 4X, '---------------------- ' )

	! Write data to INN.DAT for input to kurve-mac2.exe
		
	OPEN (UNIT=200, FILE='INN.DAT', STATUS='replace', ACTION='WRITE' &
     		&, ACCESS='sequential', IOSTAT=status)

    WRITE(200,1000,IOSTAT = readstatus) order_of_polynomial + 1

    WRITE(200,1001,IOSTAT = readstatus) initial_value

    to_file: do j = 1,no_vel_cal_points

        WRITE(200,1002,IOSTAT = readstatus)   corr_e_vel_cal(i,j) - e_0(i), vel_pitot_U(j)

    end do to_file

    CLOSE(200)

    ! Call kurve-mac2.exe to calculate polynomial coefficients based on current INN.DAT

	IF (order_of_polynomial == 3) THEN

		CALL system ('kurve-mac2-order3.exe')

    ELSE IF (order_of_polynomial == 4) THEN

		CALL system ('kurve-mac2-order4.exe')

    ELSE IF (order_of_polynomial == 2) THEN

		CALL system ('kurve-mac2-order2.exe')

    END IF

   	OPEN (UNIT=300, FILE='KURVE_OUT.DAT', STATUS='old', ACTION='READ' &
     		&, ACCESS='sequential', IOSTAT=status)

    1003 FORMAT(F14.7)     

    ! Build the coefficient matrix (coeff_mat)

    get_coef: do j = 1,order_of_polynomial + 1
  
		READ(300,1003,IOSTAT = readstatus) coeff_mat(i,j)

        if (readstatus==-1) exit   ! Checks for end of file

	end do get_coef

	CLOSE(300)

end do outer

END SUBROUTINE

! #############################################################################

SUBROUTINE write_polyfit_plotdata(filename)

! Write the polynomial coefficients, corrected anemometer output and pitot velocity
! to file, meant for import and plotting in matlab

USE variables

INTEGER                 :: i,j
CHARACTER(len=50)       :: filename
INTEGER                 :: file_status


OPEN (UNIT=100, FILE=filename, STATUS='replace', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=file_status)

!write(*,*) filename

if (order_of_polynomial == 2) then

	!WRITE(100,*)'Calibration polynomials' 
	504 FORMAT(3F14.7)
	DO j = 1,3
		WRITE(100,504) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3)
	END DO

elseif (order_of_polynomial == 3) then

	!WRITE(100,*)'Calibration polynomials' 
	505 FORMAT(4F14.7)
	DO j = 1,3
		WRITE(100,505) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4)
	END DO

elseif (order_of_polynomial == 4) then

	!WRITE(100,*)'Calibration polynomials' 
	506 FORMAT(5F14.7)
	DO j = 1,3
		WRITE(100,506) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4),coeff_mat(j,5)
	END DO

end if


!WRITE(100,*)'HW1   HW2     HW3     Pitot' 

507 FORMAT(4F14.7)

DO i = 1,no_vel_cal_points
    WRITE(100,507) corr_e_vel_cal(1,i)-e_0(1),corr_e_vel_cal(2,i)-e_0(2),corr_e_vel_cal(3,i)-e_0(3),vel_pitot_U(i)
END DO

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE write_velocity_calibration_log(filename)

! Saves data from the calibration section, which can be loaded in other parts
! of the program

USE variables

CHARACTER(len=70)			:: filename
INTEGER                     :: log_status
INTEGER 					:: j

OPEN (UNIT=100, FILE=filename, STATUS='unknown', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=log_status)

WRITE(100,*)'Wire temperatures' 
500 FORMAT(F8.3)
WRITE(100,500) T_w

WRITE(100,*)'Reference temperature' 
501 FORMAT(F8.3)
WRITE(100,501) T_0

WRITE(100,*)'Reference pitot voltage' 
502 FORMAT(F12.6)
WRITE(100,502) pitot_min_E

WRITE(100,*)'Order of polynomial' 
503 FORMAT(I1)
WRITE(100,503)order_of_polynomial

if (order_of_polynomial == 2) then

	WRITE(100,*)'Calibration polynomials' 
	504 FORMAT(3F14.7)
	DO j = 1,3
		WRITE(100,504) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3)
	END DO

elseif (order_of_polynomial == 3) then

	WRITE(100,*)'Calibration polynomials' 
	505 FORMAT(4F14.7)
	DO j = 1,3
		WRITE(100,505) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4)
	END DO

elseif (order_of_polynomial == 4) then

	WRITE(100,*)'Calibration polynomials' 
	506 FORMAT(5F14.7)
	DO j = 1,3
		WRITE(100,506) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4),coeff_mat(j,5)
	END DO

end if


WRITE(100,*)'Wire zero voltages' 
507 FORMAT(3F10.5)
WRITE(100,507) e_0(1),e_0(2),e_0(3)

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE read_velocity_calibration_log(filename)

! Reads in the data from the velocity calibration log

USE variables

CHARACTER(len=70)			:: filename
CHARACTER(len=50)			:: dummy
INTEGER                     :: log_status
INTEGER 					:: j

OPEN (UNIT=100, FILE=filename, STATUS='unknown', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=log_status)

READ(100,*) dummy
500 FORMAT(F8.3)
READ(100,500) T_w

READ(100,*) dummy 
501 FORMAT(F8.3)
READ(100,501) T_0

READ(100,*) dummy
502 FORMAT(F12.6)
READ(100,502) pitot_min_E

READ(100,*) dummy
503 FORMAT(I1)
READ(100,503)order_of_polynomial

if (order_of_polynomial == 2) then

	READ(100,*) dummy
	504 FORMAT(3F14.7)
	DO j = 1,3
		READ(100,504) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3)
	END DO

elseif (order_of_polynomial == 3) then

	READ(100,*) dummy 
	505 FORMAT(4F14.7)
	DO j = 1,3
		READ(100,505) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4)
	END DO

elseif (order_of_polynomial == 4) then

	READ(100,*) dummy 
	506 FORMAT(5F14.7)
	DO j = 1,3
		READ(100,506) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4),coeff_mat(j,5)
	END DO

end if

READ(100,*) dummy
507 FORMAT(3F10.5)
READ(100,507) e_0(1),e_0(2),e_0(3)

CLOSE(100)

END SUBROUTINE


! #############################################################################

!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$    Angle calibration    !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
 
! #############################################################################


SUBROUTINE read_effective_angle_calibration_data(filename_base)

! Imports data for calibration of the effective angles of the wires
! Allocates: e_ea_cal(3,number_of,3) -> anemometer output
!            ea_cal_pitot_E(3,number_of) -> amplified pitot output


USE variables

CHARACTER(len=70)       :: filename_base, filename
CHARACTER               :: number
INTEGER                 :: i,j
INTEGER                 :: number_of

do i = 1,3

    WRITE( number, '(I1)' )  i

    filename = TRIM(filename_base) // number // '.txt'

	write(*,*) filename

    CALL read_calibration_data(filename, number_of, angle_no_headers) 
    
    if(.not. ALLOCATED(e_ea_cal)) THEN
        ALLOCATE(e_ea_cal(3,no_angle_cal_points,3))
    end if

    if(.not. ALLOCATED(ea_cal_pitot_E)) THEN
        ALLOCATE(ea_cal_pitot_E(3,no_angle_cal_points))
    end if   

	WRITE(*,*) 'Offset voltage, eofz, wire 1-3'
	WRITE(*,*) eofz(1)
	WRITE(*,*) eofz(2)
	WRITE(*,*) eofz(3)
	WRITE(*,*)

	WRITE(*,*) 'Angle calibration data, E, wire 1-3'
	
	do j = 1,no_angle_cal_points

		WRITE(*,*) wire_E1(j),wire_E2(j),wire_E3(j)

 	    e_ea_cal(1,j,i) = (wire_E1(j)/K1) - eofz(1)
 	    e_ea_cal(2,j,i) = (wire_E2(j)/K2) - eofz(2)
   	    e_ea_cal(3,j,i) = (wire_E3(j)/K3) - eofz(3)
       
	end do
        
	WRITE(*,*)

	ea_cal_pitot_E(i,:) = pitot_E(1:no_angle_cal_points)    

end do 

DEALLOCATE(wire_E1,wire_E2,wire_E3,pitot_E)

!no_angle_cal_points = 9


END SUBROUTINE


! #############################################################################

SUBROUTINE ea_cal_temperatures(filename)

! Imports temperatures from file

! Allocates: ea_cal_T_array(3,no_angle_cal_points)

USE variables

CHARACTER(len=70)		:: filename
INTEGER					:: i,j
INTEGER					:: status

ALLOCATE(ea_cal_T_array(3,no_angle_cal_points))

OPEN (UNIT=100, FILE=filename, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

WRITE(*,*) 'Temperature for calibration points'

do j = 1,3
	do i = 1,no_angle_cal_points

		READ(100,*) ea_cal_T_array(j,i)
        
        WRITE(*,*) ea_cal_T_array(j,i)

	end do 
    	    
end do

WRITE(*,*)
        
CLOSE(100)

END SUBROUTINE 

! #############################################################################

SUBROUTINE convert_wire_voltage_to_vel(no_points)

! Converts wire voltage to velocity
! Allocates: U_hw_angle_mat(3,no_points,3)

USE variables

INTEGER      	   :: no_points
INTEGER            :: i,j,l

if (.not. ALLOCATED(U_hw_angle_mat)) then
	ALLOCATE(U_hw_angle_mat(3,no_points,3))
end if

!corr_e_ea_cal(wire,point,rotate relative to wire l)

WRITE(*,*) 'Zero voltage, e_0, wire 1-3'
WRITE(*,*) e_0(1)
WRITE(*,*) e_0(2)
WRITE(*,*) e_0(3)
WRITE(*,*)

rel_to: do l = 1,3

	wire: do i = 1,3

		angle: do j = 1,no_points

        if (order_of_polynomial == 2) then

        ! Second order polynomial 

			U_hw_angle_mat(i,j,l) =    coeff_mat(i,3)*((corr_e_ea_cal(i,j,l)-e_0(i))**2) &
                                     + coeff_mat(i,2)*((corr_e_ea_cal(i,j,l)-e_0(i))**1) &
                    			   	 + coeff_mat(i,1)        

		elseif (order_of_polynomial == 3) then

		! Third order polynomial 

			U_hw_angle_mat(i,j,l) =  coeff_mat(i,4)*((corr_e_ea_cal(i,j,l)-e_0(i))**3) &
                                     + coeff_mat(i,3)*((corr_e_ea_cal(i,j,l)-e_0(i))**2) &
                                     + coeff_mat(i,2)*((corr_e_ea_cal(i,j,l)-e_0(i))**1) &
                    			   	 + coeff_mat(i,1)

		elseif (order_of_polynomial == 4) then

		! Fourth order polynomial 

            U_hw_angle_mat(i,j,l) = coeff_mat(i,5)*((corr_e_ea_cal(i,j,l)-e_0(i))**4) &
                                     + coeff_mat(i,4)*((corr_e_ea_cal(i,j,l)-e_0(i))**3) &
                                     + coeff_mat(i,3)*((corr_e_ea_cal(i,j,l)-e_0(i))**2) &
                                     + coeff_mat(i,2)*((corr_e_ea_cal(i,j,l)-e_0(i))**1) &
                                     + coeff_mat(i,1)

        end if

    	end do

	end do

end do

END SUBROUTINE

! #############################################################################

SUBROUTINE calculate_effective_angles(no_points)

! Writes calibration angles and velocities to INN-COS.DAT, and calls kurve-mac-cos2.exe, for each wire.

USE variables

INTEGER                 :: no_coeff = 1
INTEGER                 :: no_points
REAL                    :: initial_value 
REAL                    :: axial_velocity
INTEGER                 :: status, readstatus
INTEGER                 :: i,j

initial_value = 50*(pi/180)

1000 FORMAT (I1)

1001 FORMAT (F5.3)

1002 FORMAT (F8.4, F8.4)

1004 FORMAT (/,4X,'Fitting data for wire: ', I1)

1005 FORMAT (4X, '------------------------')

1006 FORMAT (/,4X,'Effective angle of wire ', I1, ' is: ', F7.2)

! Opening a file for matlab plot data
OPEN (UNIT=400, FILE='outdata\cosinus_fit.txt', STATUS='replace', ACTION='WRITE' &
            &, ACCESS='sequential', IOSTAT=status)
    
do i = 1,3

	WRITE(*,1004) i
    WRITE(*,1005)

    axial_velocity = U_hw_angle_mat(i,5,i)  ! <===== DENNE M VURDERES!!!! 
    
    ! Open input file with logged ramp signal and hw signals
    OPEN (UNIT=200, FILE='INN-COS.DAT', STATUS='replace', ACTION='WRITE' &
            &, ACCESS='sequential', IOSTAT=status)

    WRITE(200,1000,IOSTAT = readstatus) no_coeff

	!U_hw_angle_mat(wire, angular position, rotation relative to)	

    WRITE(200,1001,IOSTAT = readstatus) initial_value

    to_file: do j = 1,no_points

        WRITE(200,1002,IOSTAT = readstatus) calibration_angles(j)*(pi/180), U_hw_angle_mat(i,j,i)/axial_velocity 

    end do to_file

    CLOSE(200)

    CALL system ('kurve-mac-cos2.exe')

    OPEN (UNIT=300, FILE='KURVE_OUT_COS.DAT', STATUS='old', ACTION='READ' &
     		&, ACCESS='sequential', IOSTAT=status)

    1003 FORMAT(F8.5)     

    ! Build the coefficient matrix (coeff_mat)

    READ(300,1003,IOSTAT = readstatus)  eff_angle(i)
    
	if (readstatus==-1) exit   ! Checks for end of file


	CLOSE(300)

    WRITE(*,1006) i,eff_angle(i)*(180/3.14159)
    
end do

! Writing data to file for plotting in matlab

1008 FORMAT(3F10.6)
1007 FORMAT(4F10.6)

WRITE(400,1008,IOSTAT = readstatus) eff_angle(1), eff_angle(2), eff_angle(3)

do j = 1,no_points	

	WRITE(400,1007,IOSTAT = readstatus) U_hw_angle_mat(1,j,1)/U_hw_angle_mat(1,5,1), &
    									U_hw_angle_mat(2,j,2)/U_hw_angle_mat(2,5,2), &
                                        U_hw_angle_mat(3,j,3)/U_hw_angle_mat(3,5,3), &
                                        calibration_angles(j)

end do
    
CLOSE(400)

END SUBROUTINE    

! #############################################################################

SUBROUTINE solve_nonlinear_eq_anglecheck()

! Uses the equation solver DNSQE to calculate the velocity vector. Uses the effective 
! angle calibration data for wire 1. Used to check that the data reduction gives 
! reasonable results.

USE variables
USE routines2

INTEGER     						:: IOPT = 2
INTEGER   							:: N = 3
DOUBLE PRECISION, DIMENSION(3) 		:: X = (/7.,1.5,1.1/)
DOUBLE PRECISION                  	:: TOL = 1.E-8
INTEGER                 			:: NPRINT = 0
INTEGER                				:: INFO
INTEGER	               				:: LWA
DOUBLE PRECISION, DIMENSION(3)		:: FVEC
DOUBLE PRECISION, DIMENSION(33) 	:: WA
INTEGER                             :: i
INTEGER                             :: log_status
REAL                                :: theta, length, error

EXTERNAL FCN


LWA = 33 !(3*N**2+13*N)/2

OPEN (UNIT=9, FILE='outdata/velocity_check.txt', STATUS='replace', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=log_status)

100 FORMAT(4X, F6.2,T16,F8.4,T26,F8.4,T36,F8.4,T46,F8.4,T60,F6.2, T71, F8.4, T82, F8.4)

WRITE(9,*) 'True angle  -  U_pitot  -    U    -    V    -    W    - Calc. angle -   |V|   - Rel. error'

WRITE(*,*) '    Angle    U_pitot    U         V         W       Theta    |V| '!  - Rel. error'

!WRITE(*,*) U_hw_angle_mat(1,5,1),U_hw_angle_mat(2,5,1),U_hw_angle_mat(3,5,1)

DO i = 1,no_angle_cal_points

	U_hw = U_hw_angle_mat(1:3,i,1)

	CALL DNSQE(FCN, FCN, IOPT, N, X, FVEC, TOL, NPRINT, INFO, WA, LWA)
    	! FCN    - is the name of the user-supplied subroutine which calculates the functions
   		! JAC    - is the name of the user-supplied subroutine which calculates the Jacobian
	    ! IOPT   - is an input variable which specifies how the Jacobian will be calculated, iopt = 2, the jacobian doesnt need to be specified.
   		! N      - is a positive integer input variable set to the number of functions and variables
    	! X      - is an array of length N.  On input X must contain an initial estimate of the solution vector. On output X contains the
    	!          final estimate of the solution vector
    	! FVEC   - is an output array of length N which contains the functions evaluated at the output X.
    	! TOL    - is a nonnegative input variable.  Termination occurs when the algorithm estimates that the relative error between X and
    	!          the solution is at most TOL
    	! NPRINT - is an integer input variable that enables controlled printing of iterates if it is positive
    	! INFO   - is an integer output variable
    	! WA     - is a work array of length LWA.
		! LWA    - is a positive integer input variable not less than (3*N**2+13*N))/2
   
	length = sqrt(X(1)**2 + X(2)**2 + X(3)**2)
    
	theta = atan(X(3)/X(1))*(180/3.14)	

    error = abs(length - pitot_U_ea(1,i))/pitot_U_ea(1,i)
	
	WRITE(9,100) calibration_angles(i), pitot_U_ea(1,i), X(1), X(2), X(3), theta, length, error 

	101 FORMAT(4X, F6.2,T13,F8.4,T22,F8.4,T32,F8.4,T42,F8.4,T53,F6.2, T60, F7.3)!, T82, F8.4)
	WRITE(*,101) calibration_angles(i),pitot_U_ea(1,i),X(1),X(2),X(3), theta, length 

END DO

CLOSE(9)

END SUBROUTINE

! #############################################################################

SUBROUTINE write_angle_calibration_log(filename)

! Saves data from the angle calibration section, which can be loaded in other parts
! of the program

USE variables

CHARACTER(len=70)			:: filename
INTEGER                     :: log_status
INTEGER 					:: j,i

OPEN (UNIT=100, FILE=filename, STATUS='unknown', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=log_status)

WRITE(100,*)'Effective angles' 
502 FORMAT(3F10.5)
WRITE(100,502) eff_angle(1),eff_angle(2),eff_angle(3)

WRITE(100,*)'Velocity data from effective angle calibration' 
WRITE(100,*)'Number of angle calibration points' 
505 FORMAT(I2)
WRITE(100,505) no_angle_cal_points

503 FORMAT(5F10.5)
504 FORMAT(' Calibration with respect to wire ',I1)
DO j = 1,3
  WRITE(100,504) j
  WRITE(100,*)'ANGLE       HW1       HW2       HW3       PITOT'
  DO i = 1,no_angle_cal_points
    WRITE(100,503) calibration_angles(i),U_hw_angle_mat(1,i,j),U_hw_angle_mat(2,i,j),U_hw_angle_mat(3,i,j),pitot_U_ea(j,i)
  END DO
END DO

WRITE(100,*)'phi_w1' 
506 FORMAT(F10.5)
WRITE(100,506) phi_w1

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE read_angle_calibration_log(filename)

! Reads in the data from the angle calibration log

USE variables

CHARACTER(len=70)			:: filename
CHARACTER(len=50)			:: dummy
INTEGER                     :: log_status
INTEGER 					:: j,i


OPEN (UNIT=100, FILE=filename, STATUS='unknown', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=log_status)


READ(100,*) dummy
502 FORMAT(3F10.5)
READ(100,502) eff_angle(1),eff_angle(2),eff_angle(3)

READ(100,*) dummy
READ(100,*) dummy
505 FORMAT(I2)
READ(100,505) no_angle_cal_points

if (.not. ALLOCATED(U_hw_angle_mat)) then
	ALLOCATE(U_hw_angle_mat(4,no_angle_cal_points,3))
end if

if (.not. ALLOCATED(pitot_U_ea)) then
	ALLOCATE(pitot_U_ea(3,no_angle_cal_points))
end if

503 FORMAT(5F10.5)
DO j = 1,3
  READ(100,*) dummy
  READ(100,*) dummy
  DO i = 1,no_angle_cal_points
    READ(100,503) calibration_angles(i),U_hw_angle_mat(1,i,j),U_hw_angle_mat(2,i,j),U_hw_angle_mat(3,i,j),pitot_U_ea(j,i)
  END DO
END DO

READ(100,*) dummy
506 FORMAT(F10.5)
READ(100,506) phi_w1

CLOSE(100)

END SUBROUTINE

! #############################################################################

!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$   Store data    !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
 
! #############################################################################

SUBROUTINE store_velocity_calibration_data(filename)

USE variables

CHARACTER(len=70)		:: filename
INTEGER                 :: status
INTEGER					:: j

! Open input file with logged ramp signal and hw signals
OPEN (UNIT=100, FILE=filename, STATUS='replace', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=status)


WRITE(100,*) 'Reference temperature'

WRITE(100,'(F6.2)') T_0 

WRITE(100,*)  'e_0'

WRITE(100,'(3F10.5)') e_0(1),e_0(2),e_0(3)

WRITE(100,*) 'eofz'
   
WRITE(100,'(3F10.5)') eofz(1),eofz(2),eofz(3)

WRITE(100,*)  'T_w'
  
WRITE(100,'(F9.4)') T_w

WRITE(100,*) 'Anemometer amplification'

WRITE(100,'(F6.2)') K1
WRITE(100,'(F6.2)') K2
WRITE(100,'(F6.2)') K3

WRITE(100,*)'Order of polynomial' 
503 FORMAT(I1)
WRITE(100,503)order_of_polynomial

if (order_of_polynomial == 2) then

	WRITE(100,*)'Calibration polynomials' 
	504 FORMAT(3F14.7)
	DO j = 1,3
		WRITE(100,504) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3)
	END DO

elseif (order_of_polynomial == 3) then

	WRITE(100,*)'Calibration polynomials' 
	505 FORMAT(4F14.7)
	DO j = 1,3
		WRITE(100,505) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4)
	END DO

elseif (order_of_polynomial == 4) then

	WRITE(100,*)'Calibration polynomials' 
	506 FORMAT(5F14.7)
	DO j = 1,3
		WRITE(100,506) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4),coeff_mat(j,5)
	END DO

end if


WRITE(100,*)  'pitot_min_E'
  
WRITE(100,'(F9.4)') pitot_min_E
    

WRITE(100,*)  'K_pitot_1'
  
WRITE(100,'(F9.4)') K_pitot_1


WRITE(100,*)  'K_pitot_2'
  
WRITE(100,'(F9.4)') K_pitot_2


WRITE(100,*)  'pitot_gain'
  
WRITE(100,'(I2)') pitot_gain

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE store_angle_data(filename)

USE variables

CHARACTER(len=70)		:: filename
INTEGER                 :: status

! Open input file with logged ramp signal and hw signals
OPEN (UNIT=100, FILE=filename, STATUS='replace', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=status)

WRITE(100,*) 'Reference rotation angle'

WRITE(100,'(F10.5)') phi_w1

WRITE(100,*) 'Effective angles'

WRITE(100,'(3F10.5)') eff_angle(1), eff_angle(2), eff_angle(3)

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE read_stored_velocity_calibration_data(filename)

USE variables

CHARACTER(len=50)		:: filename
CHARACTER(len=50) 		:: dummy
INTEGER                 :: status
INTEGER					:: j

! Open input file with logged ramp signal and hw signals
OPEN (UNIT=100, FILE=filename, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)


READ(100,*) dummy   ! Reference temperature

READ(100,'(F6.2)') T_0 

READ(100,*) dummy   ! e_0

READ(100,'(3F10.5)') e_0(1),e_0(2),e_0(3)

READ(100,*) dummy   ! eofz
   
READ(100,'(3F10.5)') eofz(1),eofz(2),eofz(3)

READ(100,*) dummy   ! T_w
  
READ(100,'(F9.4)') T_w

READ(100,*) dummy   ! anemometer amplification

READ(100,'(F6.2)') K1
READ(100,'(F6.2)') K2
READ(100,'(F6.2)') K3
  
!WRITE(*,*) K_anemometer

READ(100,*) dummy 
503 FORMAT(I1)
READ(100,503)order_of_polynomial

if (order_of_polynomial == 2) then

	READ(100,*) dummy 
	504 FORMAT(3F14.7)
	DO j = 1,3
		READ(100,504) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3)
	END DO

elseif (order_of_polynomial == 3) then

	READ(100,*) dummy 
	505 FORMAT(4F14.7)
	DO j = 1,3
		READ(100,505) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4)
	END DO

elseif (order_of_polynomial == 4) then

	READ(100,*) dummy
	506 FORMAT(5F14.7)
	DO j = 1,3
		READ(100,506) coeff_mat(j,1),coeff_mat(j,2),coeff_mat(j,3),coeff_mat(j,4),coeff_mat(j,5)
	END DO

end if

READ(100,*)  dummy
  
READ(100,'(F9.4)') pitot_min_E

READ(100,*)  dummy
  
READ(100,'(F9.4)') K_pitot_1


READ(100,*)  dummy
  
READ(100,'(F9.4)') K_pitot_2


READ(100,*)  dummy
  
READ(100,'(I2)') pitot_gain


CLOSE(100)

END SUBROUTINE

! #############################################################################


SUBROUTINE read_stored_angle_data(filename)

USE variables

CHARACTER(len=50)		:: filename
INTEGER                 :: status
CHARACTER(len=50) 		:: dummy

! Open input file with logged ramp signal and hw signals
OPEN (UNIT=100, FILE=filename, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

READ(100,*) dummy

READ(100,'(F10.5)') phi_w1

READ(100,*) dummy

READ(100,'(3F10.5)') eff_angle(1), eff_angle(2), eff_angle(3)

CLOSE(100)

END SUBROUTINE

! #############################################################################

!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$     Solve timeseries    !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
 
! #############################################################################

SUBROUTINE timeseries_temperatures(filename)

! Subroutine used to store temperatures for correction of traverse timeseries

! Allocates: traverse_T_array(no_vel_cal_points)

USE variables

CHARACTER(len=50)		:: filename
INTEGER					:: status,i

ALLOCATE(traverse_T_array(timeseries_no_points))

OPEN (UNIT=100, FILE=filename, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

do i = 1,timeseries_no_points

	READ(100,*) traverse_T_array(i)

end do 
  
CLOSE(100)

END SUBROUTINE 

! #############################################################################

SUBROUTINE read_solve_and_write_U_vector(filename_voltage_in, filename_base_point) 

! Reads in traverse data, temperature corrects it and either calulates the velocity for the 
! individual wire for export to matlab or solves the equation by calling solve_equations.

USE variables


INTEGER					 							:: status
INTEGER                                				:: i,j,k
CHARACTER(len=70),INTENT(IN) 						:: filename_voltage_in
CHARACTER(len=70),INTENT(IN) 						:: filename_base_point
CHARACTER(len=70) 				 					:: filename
CHARACTER(len=70) 				 					:: test
CHARACTER(len=1) 				 					:: number
CHARACTER(len=2) 				 					:: number2
INTEGER 											:: readstatus
INTEGER 											:: writestatus
REAL												:: E_1,E_2,E_3
REAL, DIMENSIOn(3)									:: corr_E
REAL												:: tfak 
DOUBLE PRECISION, DIMENSION(3)						:: U_vector = (/9.,0.1,0.1/)

! Writes different data to check 
WRITE(*,*) 'Anemometer amplification'
WRITE(*,*) K1,K2,K3
WRITE(*,*) 
WRITE(*,*) 'eofz'
WRITE(*,*) eofz(1),eofz(2),eofz(3)
WRITE(*,*) 
WRITE(*,*) 'Reference temperature, T_0'
WRITE(*,*) T_0
WRITE(*,*) 
WRITE(*,*) 'Wire temperature, T_w'
WRITE(*,*) T_w
WRITE(*,*) 
WRITE(*,*) 'Order of polynomial'
WRITE(*,*) order_of_polynomial
WRITE(*,*) 
WRITE(*,*) 'phi_w1'
WRITE(*,*) phi_w1
WRITE(*,*) 

! Open input file with logged hw signals
OPEN (UNIT=100, FILE=filename_voltage_in, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

!write(*,*) status
WRITE(*,*) filename_voltage_in

! Read headers

1000 FORMAT (9/)

READ(100,1000,IOSTAT = readstatus) test

!1001 FORMAT (7F10.6)
1002 FORMAT (3F10.6)

do i = 1,timeseries_no_points

	WRITE(*,*) 'Point: ',i
    no_invalid_points = 0
	
	if(i<10) then
  		WRITE( number, '(I1)' )  i
        filename = TRIM(filename_base_point) // number // '.txt'
    elseif(i>=10) then
		WRITE( number2, '(I2)' )  i
        filename = TRIM(filename_base_point) // number2 // '.txt'
	end if
  
	WRITE(*,*) filename

	! Open input file with logged ramp signal and hw signals
	OPEN (UNIT=200, FILE=filename, STATUS='replace', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=status)
   
	WRITE(*,*) traverse_T_array(i)
	
	do j = 1,timeseries_sampling_rate*timeseries_sampling_time

    	!WRITE(*,*) j

		READ(100,*,IOSTAT = readstatus) E_1, E_2, E_3
		
        e_1 = E_1/K1 - eofz(1)
        e_2 = E_2/K2 - eofz(2)
        e_3 = E_3/K3 - eofz(3)

        tfak = sqrt((T_w - T_0)/(T_w - traverse_T_array(i)))

        !WRITE(*,*) tfak
    
        corr_e(1) = e_1*tfak
        corr_e(2) = e_2*tfak
        corr_e(3) = e_3*tfak

        if (order_of_polynomial == 2) then

        ! Second order polynomial 
            do k = 1,3
            
                U_hw(k) = coeff_mat(k,3)*((corr_e(k)-e_0(k))**2) &
                          + coeff_mat(k,2)*((corr_e(k)-e_0(k))**1) &
                          + coeff_mat(k,1)    

            end do

        elseif (order_of_polynomial == 3) then

        ! Third order polynomial 
            do k = 1,3
            
                U_hw(k) = coeff_mat(k,4)*((corr_e(k)-e_0(k))**3) &
                          + coeff_mat(k,3)*((corr_e(k)-e_0(k))**2) &
                          + coeff_mat(k,2)*((corr_e(k)-e_0(k))**1) &
                          + coeff_mat(k,1)    

            end do

        elseif (order_of_polynomial == 4) then

        ! Fourth order polynomial 

            do k = 1,3
            
                U_hw(k) = coeff_mat(k,5)*((corr_e(k)-e_0(k))**4) &
                          + coeff_mat(k,4)*((corr_e(k)-e_0(k))**3) &
                          + coeff_mat(k,3)*((corr_e(k)-e_0(k))**2) &
                          + coeff_mat(k,2)*((corr_e(k)-e_0(k))**1) &
                          + coeff_mat(k,1)   

            end do

        end if

			
		!CALL solve_equations(U_vector)
        !WRITE(200,1002,IOSTAT = writestatus) U_vector(1), U_vector(2), U_vector(3)


        WRITE(200,1002,IOSTAT = writestatus) U_hw(1), U_hw(2), U_hw(3)


!$$$$$$         if(U_vector(1) < 5.)then
!$$$$$$           U_vector(1) = 10
!$$$$$$         end if
!$$$$$$ 
!$$$$$$         if(abs(U_vector(2)) > 0.7*U_vector(1))then
!$$$$$$           U_vector(2) = 0.1
!$$$$$$         end if
!$$$$$$ 
!$$$$$$         if(abs(U_vector(3)) > 0.7*U_vector(1))then
!$$$$$$           U_vector(3) = 0.1
!$$$$$$         end if
       

    end do
  
	CLOSE(200)

    READ(100,*,IOSTAT = readstatus) test   ! leser end-merket

	WRITE(*,*) 'Invalid points: ',no_invalid_points

end do

CLOSE(100)

     
END SUBROUTINE

! #############################################################################

SUBROUTINE calculate_statistics(filename_base_in,filename_statistics_out)

USE variables

CHARACTER(len=70),INTENT(IN) 		:: filename_statistics_out
CHARACTER(len=70),INTENT(IN) 		:: filename_base_in
CHARACTER(len=70)			 		:: filename
CHARACTER(len=1)                    :: number
CHARACTER(len=2)                    :: number2
INTEGER                             :: status
INTEGER     					    :: i,k
INTEGER     					    :: no
REAL, ALLOCATABLE,DIMENSION(:,:)	:: U_vector
REAL       							:: sumU,sumV,sumW
REAL								:: u_turb,v_turb,w_turb
REAL								:: uv,uw,vw 
REAL								:: uvw,u2v,u2w
REAL								:: v2u,v2w
REAL								:: w2u,w2v 
REAL								:: u3,v3,w3
REAL								:: u2,v2,w2
REAL								:: U_mean,V_mean,W_mean
REAL								:: U_temp,V_temp,W_temp




if (.not. ALLOCATED(U_vector)) then
	ALLOCATE(U_vector(timeseries_sampling_rate*timeseries_sampling_time,3))
end if


! Open output file for statistics
OPEN (UNIT=100, FILE=filename_statistics_out, STATUS='replace', ACTION='WRITE' &
      , ACCESS='sequential', IOSTAT=status)

!WRITE(100,*) 'Point   U    V    W     uv   uw    vw'
!WRITE(100,*) '-------------------------------------'
!1002 FORMAT( I2,  )

1001 FORMAT (12F10.6)

!1002 FORMAT (3F10.6)

do i = 1,11 !timeseries_no_points

	no_invalid_points = 0
    no = 0

	WRITE(*,*) i
	
	if (i<10) then
  		WRITE( number, '(I1)' )  i
        filename = TRIM(filename_base_in) // number // '.txt'
    elseif (i>=10) then
		WRITE( number2, '(I2)' )  i
        filename = TRIM(filename_base_in) // number2 // '.txt'
	end if
  
	write(*,*) filename

	! Open input file with velocity vector
	OPEN (UNIT=200, FILE=filename, STATUS='old', ACTION='read' &
     	 , ACCESS='sequential', IOSTAT=status)

	write(*,*) status
	
	sumU = 0.
	sumV = 0.        
	sumW = 0.

 	do k = 1,timeseries_sampling_rate*timeseries_sampling_time

    	READ(200,*) U_temp,V_temp,W_temp

		if(U_temp == -10.)then
			
			no_invalid_points = no_invalid_points + 1
        	  
        else

			no = no + 1			

          	U_vector(no,1) = U_temp
            U_vector(no,2) = V_temp
            U_vector(no,3) = W_temp

            sumU = sumU + U_vector(no,1)
    		sumV = sumV + U_vector(no,2)
    		sumW = sumW + U_vector(no,3)

        end if

  
	end do

	U_mean = sumU/no
	V_mean = sumV/no
	W_mean = sumW/no
    
	u2 = 0.
    v2 = 0.
    w2 = 0.
    uv = 0.
    uw = 0.
    vw = 0.
    uvw = 0.
    u2v = 0.
    u2w = 0.
    v2u = 0.
    v2w = 0.
    w2u = 0.
    w2v = 0.
    u3 = 0.
    v3 = 0.
    w3 = 0.

	do k = 1,no
    
    	u_turb = U_vector(k,1) - U_mean
   		v_turb = U_vector(k,2) - V_mean
    	w_turb = U_vector(k,3) - W_mean

   		u2 = u2 + u_turb**2
   		v2 = v2 + v_turb**2
   		w2 = w2 + w_turb**2
        uv = uv + u_turb*v_turb
        uw = uw + u_turb*w_turb
        vw = vw + v_turb*w_turb
        uvw = uvw + u_turb*v_turb*w_turb
        u2v = u2v + u_turb*u_turb*v_turb
        u2w = u2w + u_turb*u_turb*w_turb
        v2u = v2u + v_turb*v_turb*u_turb
        v2w = v2w + v_turb*v_turb*w_turb
        w2u = w2u + w_turb*w_turb*u_turb
        w2v = w2v + w_turb*w_turb*v_turb
        u3 = u3 + u_turb*u_turb*u_turb
        v3 = v3 + v_turb*v_turb*v_turb
        w3 = w3 + w_turb*w_turb*w_turb
       
	end do

	u2 = u2/no
	v2 = v2/no
	w2 = w2/no
    uv = uv/no
    uw = uw/no
    vw = vw/no
    uvw = uvw/no
    u2v = u2v/no
    u2w = u2w/no
    v2u = v2u/no
    v2w = v2w/no
    w2u = w2u/no
    w2v = w2v/no
    u3 = u3/no
    v3 = v3/no
    w3 = w3/no

	CLOSE(200)

	WRITE(100,1001) U_mean, V_mean, W_mean , uv, uw, vw , u2,v2,w2,u2w, v2W, w3

	write(*,*) 'Number of invalid points:', no_invalid_points 

end do

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE solve_meanvalues(filename_voltage_in,filename_statistics_mean) 

! Allocates: E
! Defines: no_samples

USE variables
!USE routines2

INTEGER					 							:: status
INTEGER                                				:: j,k
CHARACTER(len=50),INTENT(IN) 						:: filename_voltage_in
CHARACTER(len=50),INTENT(IN) 						:: filename_statistics_mean
CHARACTER(len=50) 				 					:: test
INTEGER 											:: readstatus
INTEGER 											:: writestatus
REAL												:: E_1,E_2,E_3
REAL, DIMENSION(3)									:: corr_E
REAL												:: tfak 
DOUBLE PRECISION, DIMENSION(3)						:: U_vector = (/9.,0.1,0.1/)
REAL,DIMENSION(12) 									:: T
REAL												:: E_pitot
REAL												:: dP_pitot
REAL												:: V_pitot

T = (/23.4,23.4,23.4,23.4,23.4,23.4,23.5,23.5,23.5,23.5,23.5,23.5/)
!T = (/20.8,22.1,22.0,21.9/)

!DEALLOCATE(E_amplifier)

! Open input file with logged hw signals
OPEN (UNIT=100, FILE=filename_voltage_in, STATUS='old', ACTION='READ' &
     &, ACCESS='sequential', IOSTAT=status)

write(*,*) filename_voltage_in

! Open output for statistics
OPEN (UNIT=200, FILE=filename_statistics_mean, STATUS='replace', ACTION='WRITE' &
     &, ACCESS='sequential', IOSTAT=status)


! Read headers

1000 FORMAT (10/)

READ(100,1000,IOSTAT = readstatus) test

1001 FORMAT (4F12.6)
	
do j = 1,12

  	no_invalid_points = 0

   	WRITE(*,*) j

	READ(100,*,IOSTAT = readstatus) E_1, E_2, E_3, E_pitot
		
    e_1 = (E_1/K1) - eofz(1)
    e_2 = (E_2/K2) - eofz(2)
    e_3 = (E_3/K3) - eofz(3)

    dP_pitot = (((E_pitot-pitot_min_E)/pitot_gain)*K_pitot_1 + K_pitot_2)
    V_pitot = sqrt(dP_pitot/(rho*0.5))

    !tfak = sqrt((T_w - T_0)/(T_w - traverse_T_array(j)))
    tfak = sqrt((T_w - T_0)/(T_w - T(j)))

    corr_e(1) = e_1*tfak
    corr_e(2) = e_2*tfak
    corr_e(3) = e_3*tfak

    if (order_of_polynomial == 2) then

    	! Second order polynomial 
    	do k = 1,3
            
      	       U_hw(k) = coeff_mat(k,3)*((corr_e(k)-e_0(k))**2) &
     	                 + coeff_mat(k,2)*((corr_e(k)-e_0(k))**1) &
      	                 + coeff_mat(k,1)    
 	   	end do

   	elseif (order_of_polynomial == 3) then

        ! Third order polynomial 
        do k = 1,3
            
       	      U_hw(k) = coeff_mat(k,4)*((corr_e(k)-e_0(k))**3) &
        	            + coeff_mat(k,3)*((corr_e(k)-e_0(k))**2) &
                        + coeff_mat(k,2)*((corr_e(k)-e_0(k))**1) &
                        + coeff_mat(k,1)    

        end do

   	elseif (order_of_polynomial == 4) then

        ! Fourth order polynomial 
		do k = 1,3
            
              U_hw(k) = coeff_mat(k,5)*((corr_e(k)-e_0(k))**4) &
                        + coeff_mat(k,4)*((corr_e(k)-e_0(k))**3) &
                        + coeff_mat(k,3)*((corr_e(k)-e_0(k))**2) &
                        + coeff_mat(k,2)*((corr_e(k)-e_0(k))**1) &
                        + coeff_mat(k,1)   

         end do

    end if
	
	CALL solve_equations(U_vector)
        
    WRITE(200,1001,IOSTAT = writestatus) U_vector(1), U_vector(2), U_vector(3), V_pitot
   
	!write(*,*) writestatus

end do

CLOSE(200)
  
CLOSE(100)

     
END SUBROUTINE

! #############################################################################

SUBROUTINE conditional_average(filename_base_in,filename_cond_average)

USE variables

CHARACTER(len=70),INTENT(IN) 		:: filename_cond_average
CHARACTER(len=70),INTENT(IN) 		:: filename_base_in
CHARACTER(len=70)			 		:: filename
CHARACTER(len=1)                    :: number
CHARACTER(len=2)                    :: number2
INTEGER                             :: status
INTEGER     					    :: i,j,k,m
REAL, ALLOCATABLE,DIMENSION(:,:)	:: U_vector
REAL								:: shedding_frequency
REAL								:: period
REAL								:: dt_bin
REAL								:: dt
REAL								:: t
INTEGER								:: index
INTEGER, DIMENSION(100)			    :: bin_no
REAL, DIMENSION(100,6000)			:: U_array
REAL, DIMENSION(100,6000)			:: V_array
REAL, DIMENSION(100,6000)			:: W_array
REAL       							:: sumU,sumV,sumW
REAL								:: u_turb,v_turb,w_turb
REAL								:: uv,uw,vw 
REAL								:: u2,v2,w2
REAL								:: U_mean,V_mean,W_mean
REAL								:: bin_U_mean,bin_V_mean,bin_W_mean
INTEGER								:: no
INTEGER								:: no_bins = 50
REAL								:: minimum,maximum


OPEN (UNIT=100, FILE=filename_cond_average, STATUS='replace', ACTION='WRITE' &
      , ACCESS='sequential', IOSTAT=status)

write(*,*) filename_cond_average
  
if (.not. ALLOCATED(U_vector)) then
	ALLOCATE(U_vector(timeseries_sampling_rate*timeseries_sampling_time,3))
end if

!shedding_frequency = 108.3
!shedding_frequency = 42.05


1002 FORMAT (9F10.6)


do m = 1,1

  	!shedding_frequency = 0.5*41.26 + 0.01*(m-1)
    shedding_frequency = 0.5*42.85 + 0.01*(m-1)


  	period = 1/shedding_frequency

  	dt_bin = period/no_bins

  	dt = 1./timeseries_sampling_rate

  	do i = 1,43!timeseries_no_points

		do j = 1,no_bins
  
			bin_no(j) = 0

		end do

		WRITE(*,*) i
	
		if (i<10) then
  			WRITE( number, '(I1)' )  i
     	   filename = TRIM(filename_base_in) // number // '.txt'
    	elseif (i>=10) then
			WRITE( number2, '(I2)' )  i
     	   filename = TRIM(filename_base_in) // number2 // '.txt'
		end if
  
		write(*,*) filename

		! Open input file with velocity vector
		OPEN (UNIT=200, FILE=filename, STATUS='old', ACTION='read' &
     		 , ACCESS='sequential', IOSTAT=status)

		!write(*,*) status
	
		no = timeseries_sampling_rate*timeseries_sampling_time/(20*2)
		
		sumU = 0.
    	sumV = 0.
    	sumW = 0.

 		do k = 1,no

    		READ(200,*) U_vector(k,1),U_vector(k,2),U_vector(k,3)
		
			!write(*,*) k	

			t = (k-1)*dt

        	index = floor((t - floor(t/period)*period)/dt_bin) + 1

			bin_no(index) = bin_no(index) + 1 

        	U_array(index,bin_no(index)) = U_vector(k,1)
        	V_array(index,bin_no(index)) = U_vector(k,2)
        	W_array(index,bin_no(index)) = U_vector(k,3)

        	sumU = sumU + U_vector(k,1)
    		sumV = sumV + U_vector(k,2)
    		sumW = sumW + U_vector(k,3)


   		end do
   		!write(*,*) 'here2'
 
   		CLOSE(200)

  		U_mean = sumU/no
   		V_mean = sumV/no
   		W_mean = sumW/no
   
   		u2 = 0.
   		v2 = 0.
   		w2 = 0.
   		uv = 0.
   		uw = 0.
   		vw = 0.


		maximum = -5.
        minimum = 5.

   		do k = 1,no_bins   ! Runs through the different bins
       
      	 	!WRITE(*,*) k, bin_no(k) 

       		bin_U_mean = 0.
       		bin_V_mean = 0.
       		bin_W_mean = 0.

       		do j = 1,bin_no(k) 

           		bin_U_mean = bin_U_mean + U_array(k,j)
           		bin_V_mean = bin_V_mean + V_array(k,j)
           		bin_W_mean = bin_W_mean + W_array(k,j)
   
           		u_turb = U_array(k,j) - U_mean
           		v_turb = V_array(k,j) - V_mean
           		w_turb = W_array(k,j) - W_mean
                       
           		u2 = u2 + u_turb**2
           		v2 = v2 + v_turb**2
           		w2 = w2 + w_turb**2
           		uv = uv + u_turb*v_turb
           		uw = uw + u_turb*w_turb
           		vw = vw + v_turb*w_turb

       		end do
       
       		bin_U_mean = bin_U_mean/bin_no(k)
       		bin_V_mean = bin_V_mean/bin_no(k)
       		bin_W_mean = bin_W_mean/bin_no(k)
       
       		u2 = u2/bin_no(k)
       		v2 = v2/bin_no(k)
       		w2 = w2/bin_no(k)
       		uv = uv/bin_no(k)
       		uw = uw/bin_no(k)
       		vw = vw/bin_no(k)

       		WRITE(100,1002) bin_U_mean, bin_V_mean, bin_W_mean , uv, uw, vw , u2,v2,w2
       
			if(bin_V_mean > maximum)then
            	maximum = bin_V_mean
            else if(bin_V_mean < minimum)then
              	minimum = bin_V_mean
            end if
                

   		end do

   		WRITE(100,1002) U_mean, V_mean, W_mean , uv, uw, vw , u2,v2,w2

  		CLOSE(200)

  		write(*,*) shedding_frequency,minimum,maximum,maximum-minimum

  	end do

end do

CLOSE(100)

END SUBROUTINE

! #############################################################################

SUBROUTINE solve_equations(X)

! Uses the equation solver DNSQE to calculate the velocity vector. 
! Placed in separate module because FCN needs to be external....? 

USE variables
USE routines2

INTEGER     						:: IOPT = 2
INTEGER   							:: N = 3
DOUBLE PRECISION, DIMENSION(3) 		:: X ! = (/0.01,0.01,0.01/)
DOUBLE PRECISION                  	:: TOL = 1.E-8
INTEGER                 			:: NPRINT = 1
INTEGER                				:: INFO
INTEGER	               				:: LWA
DOUBLE PRECISION, DIMENSION(3)		:: FVEC
DOUBLE PRECISION, DIMENSION(100) 	:: WA
!INTEGER                             :: i
!DOUBLE PRECISION, DIMENSION(3) 		:: V


EXTERNAL FCN

!write(*,*) X

LWA = 100  !(3*N**2+13*N)/2

!V = X

CALL DNSQE(FCN, FCN, IOPT, N, X, FVEC, TOL, NPRINT, INFO, WA, LWA)
       ! FCN    - is the name of the user-supplied subroutine which calculates the functions
       ! JAC    - is the name of the user-supplied subroutine which calculates the Jacobian
       ! IOPT   - is an input variable which specifies how the Jacobian will be calculated, iopt = 2, the jacobian doesnt need to be specified.
       ! N      - is a positive integer input variable set to the number of functions and variables
       ! X      - is an array of length N.  On input X must contain an initial estimate of the solution vector. On output X contains the
       !          final estimate of the solution vector
       ! FVEC   - is an output array of length N which contains the functions evaluated at the output X.
       ! TOL    - is a nonnegative input variable.  Termination occurs when the algorithm estimates that the relative error between X and
       !          the solution is at most TOL
       ! NPRINT - is an integer input variable that enables controlled printing of iterates if it is positive
       ! INFO   - is an integer output variable
       ! WA     - is a work array of length LWA.
       ! LWA    - is a positive integer input variable not less than (3*N**2+13*N))/2

!V = X

if (info /= 1) then
!$$$$$$     WRITE(*,*) 'INFO'
!$$$$$$     WRITE(*,*) info
!$$$$$$     WRITE(*,*) V
!$$$$$$     WRITE(*,*) FVEC    
!$$$$$$     WRITE(*,*) X
   X = (/-10.,-10.,-10./)
   no_invalid_points = no_invalid_points + 1
!$$$$$$     info = 1
!$$$$$$     WRITE(*,*) info
end if

END SUBROUTINE


! #############################################################################

SUBROUTINE settings_check()

USE variables

WRITE(*,*) 'eofz'
WRITE(*,*) eofz
WRITE(*,*) 'K1'
WRITE(*,*) K1
WRITE(*,*) 'K2'
WRITE(*,*) K2
WRITE(*,*) 'K3'
WRITE(*,*) K3
WRITE(*,*) 'K_pitot_1'
WRITE(*,*) K_pitot_1
WRITE(*,*) 'K_pitot_2'
WRITE(*,*) K_pitot_2
WRITE(*,*) 'T_0'
WRITE(*,*) T_0

END SUBROUTINE

! #############################################################################

!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$
!$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$ !$$$$$$

! #############################################################################

END MODULE


! #############################################################################

PROGRAM main

USE routines
USE routines2
USE variables

IMPLICIT NONE

!CHARACTER(len=30)		:: dummy
CHARACTER(len=70)		:: folder
INTEGER 				:: choice,subchoice
INTEGER                 :: i,j,l
INTEGER                 :: log_status

CALL filedata()

! #############################################################################

OPEN (UNIT=3, FILE=filename_log, STATUS='unknown', ACTION='WRITE' &
       &, ACCESS='sequential', IOSTAT=log_status)

! #############################################################################



! #############################################################################

!$$$$$$ OPEN (UNIT=5, FILE=polynomial_file, STATUS='unknown', ACTION='READ' &
!$$$$$$      &, ACCESS='sequential')
!$$$$$$ 
!$$$$$$ READ(5,*) order_of_polynomial  
!$$$$$$ 
!$$$$$$ CLOSE(5)

! #############################################################################

MENU: DO

WRITE(*,*)
WRITE(*,*) '    MENU'
WRITE(*,*) '--------------------------------'
WRITE(*,*) '1  - Perform velocity calibration'
WRITE(*,*) '2  - Calculate effective angles'
WRITE(*,*) '3  - Test solution on angular calibration dataset for wire 1'
WRITE(*,*) '4  - Update stored velocity calibration polynomials'
WRITE(*,*) '5  - Update stored effective angles'
WRITE(*,*) '6  - Solve timeseries and write U_vector to file'
WRITE(*,*) '7  - Calculate statistics for the timeseries'
WRITE(*,*) '8  - Solve meanprofile'
WRITE(*,*) '9  - Conditional average'
WRITE(*,*) '10 - Settings check'
WRITE(*,*) '11 - Exit'
WRITE(*,*)

READ(*,*) choice

MENU: SELECT CASE(choice)

CASE(1) ! Velocity calibration

! ####################################################################################

WRITE(*,*)
WRITE(*,*) '	    SUBMENU'
WRITE(*,*) '	--------------------------------'
WRITE(*,*) '    Perform velocity calibration for:'
WRITE(*,*) '	1 - Angle calibration'
WRITE(*,*) '	2 - Timeseries'
WRITE(*,*)

READ(*,*) subchoice

SUBMENU: SELECT CASE(subchoice)

CASE(1) ! Angle calibration

   folder = angle_calibration_folder

CASE(2) ! Timeseries

   folder = timeseries_folder

END SELECT    

!write(*,*) trim(folder) // filename_settings

! Loads settings
CALL settings(trim(folder) // filename_settings)

WRITE(*,*) '-> Enter order of polynomial fit: '
READ(*,'(I1)') order_of_polynomial


WRITE(*,*) '-> Reads velocity calibration data from file'
CALL read_velocity_calibration_data(trim(folder) // filename_velocity_calibration)
! ALLOCATES and BUILDS: e_vel_cal(3,no_vel_cal_points) 
!  						vel_cal_pitot_E(no_vel_cal_points) 
!						pitot_min


! ######## LOG ########
1001 FORMAT(3F8.4)
WRITE(3,*) 'e_vel_cal'
do i = 1,no_vel_cal_points
   WRITE(3,1001) e_vel_cal(1,i),e_vel_cal(2,i),e_vel_cal(3,i)
end do
! #####################


CALL vel_cal_temperatures(trim(folder) // filename_velocity_calibration_temperatures)
! ALLOCATES and BUILDS: vel_cal_T_array(no_vel_cal_points)
!                       T_0


!$$$$$$ WRITE(*,467)
!$$$$$$ 467 FORMAT( /,1X, '-> Calculates wire temperature',/ )
!$$$$$$ CALL calculate_Tw()  
!$$$$$$ ! Calculates and saves value in: T_w 


WRITE(*,468)
468 FORMAT( /,1X, '-> Corrects velocity calibration data for temperature change' )
CALL temperature_corrections(1)
! ALLOCATES and BUILDS: corr_e_vel_cal(3,no_vel_cal_points)


! ######## LOG ########
1002 FORMAT(3F8.4)
WRITE(3,*) 'corr_e_vel_cal'
do i = 1,no_vel_cal_points
   WRITE(3,1002) corr_e_vel_cal(1,i),corr_e_vel_cal(2,i),corr_e_vel_cal(3,i)
end do
! #####################


! ######## LOG ########
1007 FORMAT(3F8.4)
WRITE(3,*) 'corr_E_vel_cal'
do i = 1,no_vel_cal_points
   WRITE(3,1007) K1*(corr_e_vel_cal(1,i)+eofz(1)),K2*(corr_e_vel_cal(2,i)+eofz(2)),K3*(corr_e_vel_cal(3,i)+eofz(3))
end do
! #####################


WRITE(*,469)
469 FORMAT( /,1X, '-> Converts voltage output from pressure transducer ',/,4X, 'to velocity for velocity calibration' )
CALL pitot_E2U(vel_cal_pitot_E,no_vel_cal_points)
! ALLOCATES and BUILDS: pitot_U for a given vector, send in as a dummy argument along with the vector length 

if (.not. ALLOCATED(vel_pitot_U)) then
   ALLOCATE(vel_pitot_U(no_vel_cal_points))
end if

vel_pitot_U = pitot_U 
! Copies the information from pitot_U to a vector specific for the velocity calibration data

! ######## LOG ########
1003 FORMAT(1F8.4)
WRITE(3,*) 'vel_pitot_U'
WRITE(3,1003) vel_pitot_U
! #####################


DEALLOCATE(pitot_U)


WRITE(*,470)
470 FORMAT( /,1X, '-> Finds polynomial fit for the data points' )
CALL find_polynomials()
! Uses corr_e_vel_cal and vel_pitot_U
! Calculates coeff_mat [5]

WRITE(*,465)
465 FORMAT( /,1X, '-> Writes data for polynomial fit to file' )
CALL write_polyfit_plotdata(filename_polyfit)


WRITE(*,466) filename_velocity_calibration_log
466 FORMAT( /,1X, '-> Velocity calibration data is written to: ',A40/ )
CALL write_velocity_calibration_log(filename_velocity_calibration_log)


DEALLOCATE(e_vel_cal,vel_cal_pitot_E,vel_cal_T_array,corr_e_vel_cal,vel_pitot_U)


! ####################################################################################

CASE(2)	! Effective angle calibration

! ####################################################################################


WRITE(*,*) '-> Loads velocity calibration data'
CALL read_velocity_calibration_log(filename_velocity_calibration_log)

! Reads in: Wire temperatures, 				T_w
!           Reference temperature,			T_0
!           Pitot zero velocity E, 			pitot_min_E
!           Polynomial fit coefficients, 	coeff_mat(3,5)
!           Hot wire zero velocity voltage, E_0


! Loads settings
CALL settings(trim(angle_calibration_folder) // filename_settings)


WRITE(*,471)
471 FORMAT( /,1X, '-> Reads effective angle calibration data from file' )
CALL read_effective_angle_calibration_data(trim(angle_calibration_folder) // filename_angle_calibration_base )
! ALLOCATES and BUILDS: e_ea_cal(3,no_angle_cal_points,rotation relative to wire x)

! ######## LOG ########
1004 FORMAT(3F8.4)
WRITE(3,*) 'e_ea_cal'
do j = 1,3
 WRITE(3,*) j
   do i = 1,no_angle_cal_points
       WRITE(3,1004) e_ea_cal(1,i,j),e_ea_cal(2,i,j),e_ea_cal(3,i,j)
   end do
end do
! #####################


CALL ea_cal_temperatures(trim(angle_calibration_folder) // filename_angle_calibration_temperatures) ! Builds array with temperatures
! ALLOCATES and BUILDS: angle_cal_temp_array(no_angle_cal_points)
! -> Stores temperature data for effective angle calibration 

WRITE(*,475)
475 FORMAT( /,1X, '-> Corrects angle calibration data for temperature change' )
CALL temperature_corrections(2)
! ALLOCATES and BUILDS: corr_e_ea_cal(3,no_angle_cal_points,3)
! -> Stores corrected wire voltages for angle calibration 


! ######## LOG ########
1005 FORMAT(3F8.4)
WRITE(3,*) 'corr_e_ea_cal'
do j = 1,3
 WRITE(3,*) j
   do i = 1,no_angle_cal_points
       WRITE(3,1005) corr_e_ea_cal(1,i,j),corr_e_ea_cal(2,i,j),corr_e_ea_cal(3,i,j)
   end do
end do
! #####################


! ######## LOG ########
1008 FORMAT(3F8.4)
WRITE(3,*) 'corr_E_ea_cal'
do j = 1,3
   WRITE(3,*) j
   do i = 1,no_angle_cal_points
       WRITE(3,1008) K1*(corr_e_ea_cal(1,i,j)+eofz(1)),K2*(corr_e_ea_cal(2,i,j)+eofz(2)), & 
                     K3*(corr_e_ea_cal(3,i,j)+eofz(3))
   end do
end do
! #####################


!write(*,*) corr_e_ea_cal(1,1,1)-e_0(1),corr_e_ea_cal(2,1,1)-e_0(2),corr_e_ea_cal(3,1,1)-e_0(3)


WRITE(*,473)
473 FORMAT( /,1X, '-> Converting wire voltages to effective cooling velocity' )
CALL convert_wire_voltage_to_vel(no_angle_cal_points)
! ALLOCATES and BUILDS: U_hw_angle_mat(3,no_points,3)
! -> Stores effective velocity data converted from voltage based on polynomial coefficients
! -> Fourth row in matrix is unused, room for pitot axial velocity


! ######## LOG ########
1009 FORMAT(3F10.6)
WRITE(3,*) 'U_hw_angle_mat'
do j = 1,3
   WRITE(3,*) j
   do i = 1,no_angle_cal_points
       WRITE(3,1009) U_hw_angle_mat(1,i,j),U_hw_angle_mat(2,i,j),U_hw_angle_mat(3,i,j)
   end do
end do
! #####################


ALLOCATE(pitot_U_ea(3,no_angle_cal_points))


WRITE(*,472)
472 FORMAT( /,1X, '-> Converts voltage output from pressure transducer ',/,4X, &
           'to velocity for effective angle calibration' )

do l = 1,3

   CALL pitot_E2U(ea_cal_pitot_E(l,:),no_angle_cal_points)
   ! ALLOCATES and BUILDS: pitot_U for a given vector, send in as a dummy argument along with the vector length  

   pitot_U_ea(l,:) = pitot_U
   ! Stores the pitot velocity in U_hw_angle_mat

   DEALLOCATE(pitot_U)

end do


! ######## LOG ########
1006 FORMAT(3F8.4)
WRITE(3,*) 'pitot_U_ea'
do i = 1,no_angle_cal_points
   WRITE(3,1006) pitot_U_ea(1,i),pitot_U_ea(2,i),pitot_U_ea(3,i)
end do
! #####################


WRITE(*,474)
474 FORMAT( /,1X, '-> Curve fitting data to find the effective angles of the wires' )
CALL calculate_effective_angles(no_angle_cal_points)
! Calculates and stores the effective angle in eff_angle


WRITE(*,464) filename_eff_angle_calibration_log
464 FORMAT( /,1X, '-> Angle calibration data is written to: ',A40/ )
CALL write_angle_calibration_log(filename_eff_angle_calibration_log)

DEALLOCATE(e_ea_cal,corr_e_ea_cal,U_hw_angle_mat,pitot_U_ea,ea_cal_T_array)

! ####################################################################################

CASE(3)	! Solve equations

! ####################################################################################

WRITE(*,*) '-> Loads velocity calibration data'
CALL read_velocity_calibration_log(filename_velocity_calibration_log)

! Reads in: Wire temperatures, 				T_w
!           Reference temperature,			T_0
!           Pitot zero velocity E, 			pitot_min_E
!           Polynomial fit coefficients, 	coeff_mat(3,5)
!           Hot wire zero velocity voltage, E_0

WRITE(*,*) '-> Loads effective angle calibration data'
CALL read_angle_calibration_log(filename_eff_angle_calibration_log)

! Reads in: Effecive angles, 					eff_angles(3)
!           Number points for ea calibration,	no_angle_cal_points
!           Effective velocities for the wires,	U_hw_angle_mat(allocated)  
!           Pitot velocities,                   pitot_U_ea(allocated)

! Loads settings
CALL settings(trim(angle_calibration_folder) // filename_settings)

WRITE(*,*) '-> Set wire 1 rotation(angle phi): '
READ(*,'(F6.2)') phi_w1

CALL solve_nonlinear_eq_anglecheck()

DEALLOCATE(U_hw_angle_mat,pitot_U_ea)

! ###################################################################################

CASE(4)

! ####################################################################################

CALL read_velocity_calibration_log(filename_velocity_calibration_log)

CALL read_angle_calibration_log(filename_eff_angle_calibration_log)

CALL store_velocity_calibration_data(filename_stored_velocity_calibration)

DEALLOCATE(U_hw_angle_mat,pitot_U_ea)

WRITE(*,*) '-> Stored velocity calibration updated'

! ####################################################################################

CASE(5)

! ####################################################################################

CALL read_velocity_calibration_log(filename_velocity_calibration_log)

CALL read_angle_calibration_log(filename_eff_angle_calibration_log)

CALL store_angle_data(filename_stored_angles)

DEALLOCATE(U_hw_angle_mat,pitot_U_ea)

WRITE(*,*) '-> Stored angle calibration updated'

! ####################################################################################

CASE(6) ! Solve timeseries

! ####################################################################################

WRITE(*,*) '-> Reads velocity calibration data from file'
CALL read_stored_velocity_calibration_data(filename_stored_velocity_calibration)

WRITE(*,*) '-> Reads angle calibration data from file'
CALL read_stored_angle_data(filename_stored_angles)

! Loads settings
CALL settings(trim(timeseries_folder) // filename_settings)

WRITE(*,*) '-> Reads temperature data from file'
CALL timeseries_temperatures(trim(timeseries_folder) // filename_timeseries_temperature)

WRITE(*,*) '-> Reads voltage timeseries from file and solves for U,V and W'
CALL read_solve_and_write_U_vector(trim(timeseries_folder) // filename_timeseries, filename_U_vector_base)


! ####################################################################################

CASE(7)	! Calculate statistics of timeseries

! ####################################################################################


WRITE(*,*) '-> Calculating statistics of timeseries'
CALL calculate_statistics(filename_U_vector_base,filename_statistics)
     
! ####################################################################################

CASE(8)	! Solve for meanvalues

! ####################################################################################

WRITE(*,*) '-> Reads velocity calibration data from file'
CALL read_stored_velocity_calibration_data(filename_stored_velocity_calibration)

WRITE(*,*) '-> Reads angle calibration data from file'
CALL read_stored_angle_data(filename_stored_angles)

CALL timeseries_temperatures(trim(timeseries_folder) // filename_timeseries_temperature)

phi_w1 = 90.

! Loads settings
CALL settings(trim(timeseries_folder) // filename_settings)

WRITE(*,*) '-> Reads voltage meanvalues from file and solves for U,V and W'
CALL solve_meanvalues(filename_meanprofile,filename_meanprofile_statistics)

! ####################################################################################

CASE(9)

! ####################################################################################

CALL conditional_average(filename_U_vector_base,filename_conditional_average)

! ####################################################################################

CASE(10)

! ####################################################################################

WRITE(*,*) '-> Current settings:'
CALL settings_check()

! ####################################################################################

CASE(11)

! ####################################################################################

WRITE(*,*) 'Exiting program'

EXIT

! ####################################################################################

END SELECT

END DO MENU

CLOSE(3)

STOP

END PROGRAM



